@@ -1,8 +1,10 @@
 from typing import List
+from http import HTTPStatus
 
 from kentik_api.api_calls import sites
 from kentik_api.api_resources.base_api import BaseAPI
 from kentik_api.public.site import Site
+from kentik_api.public.types import ID
 from kentik_api.requests_payload import sites_payload
 
 
@@ -14,7 +16,7 @@ class SitesAPI(BaseAPI):
         response = self.send(apicall)
         return sites_payload.GetAllResponse.from_json(response.text).to_sites()
 
-    def get(self, site_id: int) -> Site:
+    def get(self, site_id: ID) -> Site:
         apicall = sites.get_site_info(site_id)
         response = self.send(apicall)
         return sites_payload.GetResponse.from_json(response.text).to_site()
@@ -32,7 +34,7 @@ class SitesAPI(BaseAPI):
         response = self.send(apicall, payload)
         return sites_payload.UpdateResponse.from_json(response.text).to_site()
 
-    def delete(self, site_id: int) -> bool:
+    def delete(self, site_id: ID) -> bool:
         apicall = sites.delete_site(site_id)
         response = self.send(apicall)
-        return response.http_status_code == 204
+        return response.http_status_code == HTTPStatus.NO_CONTENT
