@@ -1,31 +1,35 @@
-from typing import List, Dict, Optional, Any
 from dataclasses import dataclass
-from kentik_api.public.types import ID
+from typing import Any, Dict, List, Optional
+
+from kentik_api.public.device import (
+    AllInterfaces,
+    AppliedLabels,
+    AuthenticationProtocol,
+    CDNAttribute,
+    Device,
+    DeviceBGPType,
+    DeviceSubtype,
+    DeviceType,
+    PrivacyProtocol,
+    SNMPv3Conf,
+)
 from kentik_api.public.device_label import DeviceLabel
 from kentik_api.public.errors import IncompleteObjectError
 from kentik_api.public.site import Site
+from kentik_api.public.types import ID
+from kentik_api.requests_payload.conversions import (
+    convert,
+    convert_list_or_none,
+    convert_or_none,
+    dict_from_json,
+    enum_to_str,
+    from_dict,
+    list_from_json,
+    permissive_enum_to_str,
+)
 from kentik_api.requests_payload.plans_payload import GetResponse as PlanGetResponse
 from kentik_api.requests_payload.validation import validate_fields
-from kentik_api.requests_payload.conversions import (
-    from_dict,
-    from_json,
-    convert,
-    convert_or_none,
-    convert_list_or_none,
-    enum_to_str,
-)
-from kentik_api.public.device import (
-    Device,
-    DeviceType,
-    DeviceSubtype,
-    DeviceBGPType,
-    SNMPv3Conf,
-    PrivacyProtocol,
-    AuthenticationProtocol,
-    CDNAttribute,
-    AllInterfaces,
-    AppliedLabels,
-)
+
 
 # pylint: disable=too-many-instance-attributes
 
@@ -48,9 +52,9 @@ class SNMPv3ConfPayload:
     def from_conf(cls, conf: SNMPv3Conf):
         return cls(
             UserName=conf.user_name,
-            AuthenticationProtocol=convert_or_none(conf.authentication_protocol, enum_to_str),
+            AuthenticationProtocol=convert_or_none(conf.authentication_protocol, permissive_enum_to_str),
             AuthenticationPassphrase=conf.authentication_passphrase,
-            PrivacyProtocol=convert_or_none(conf.privacy_protocol, enum_to_str),
+            PrivacyProtocol=convert_or_none(conf.privacy_protocol, permissive_enum_to_str),
             PrivacyPassphrase=conf.privacy_passphrase,
         )
 
@@ -66,12 +70,16 @@ class SNMPv3ConfPayload:
 
 @dataclass
 class LabelPayload:
-    """ This datastructure represents JSON Device.Label payload as it is transmitted to and from KentikAPI """
+    """
+    This datastructure represents JSON Device.Label payload as it is transmitted to and from KentikAPI
+    LabelPayload embedded under Device differs from standalone LabelPayload in that it lacks devices list, and in field names,
+    eg. cdate vs created_date, edate vs updated_date
+    """
 
     id: int
     color: str
     name: str
-    user_id: str
+    user_id: Optional[str]  # yes, user_id can be None
     company_id: str
     cdate: str
     edate: str
@@ -82,22 +90,25 @@ class LabelPayload:
 
     def to_device_label(self) -> DeviceLabel:
         return DeviceLabel(
-            id=convert(self.id, ID),
             color=self.color,
             name=self.name,
-            user_id=convert(self.user_id, ID),
-            company_id=convert(self.company_id, ID),
-            created_date=self.cdate,
-            updated_date=self.edate,
-            devices=None,
+            _id=convert(self.id, ID),
+            _user_id=convert_or_none(self.user_id, ID),
+            _company_id=convert(self.company_id, ID),
+            _created_date=self.cdate,
+            _updated_date=self.edate,
+            _devices=[],
         )
 
 
 @dataclass
 class SitePayload:
-    """ This datastructure represents JSON Device.Site payload as it is transmitted to and from KentikAPI """
+    """
+    This datastructure represents JSON Device.Site payload as it is transmitted to and from KentikAPI.
+    SitePayload embeddedd under Device differs from regular SitePayload in that all fields are optional.
+    """
 
-    id: int
+    id: Optional[int]
     site_name: Optional[str]
     lat: Optional[float]
     lon: Optional[float]
@@ -112,7 +123,7 @@ class SitePayload:
             site_name=self.site_name,
             latitude=self.lat,
             longitude=self.lon,
-            id=convert(self.id, ID),
+            id=convert_or_none(self.id, ID),
             company_id=convert_or_none(self.company_id, ID),
         )
 
@@ -149,7 +160,7 @@ class DevicePayload:
     device_name: Optional[str] = None
     device_type: Optional[str] = None
     device_subtype: Optional[str] = None
-    device_sample_rate: Optional[int] = None
+    device_sample_rate: Optional[str] = None
     sending_ips: Optional[List[str]] = None
     id: Optional[str] = None
     plan: Optional[PlanPayload] = None
@@ -239,15 +250,15 @@ class DevicePayload:
             plan_id=convert_or_none(device.plan_id, int),
             site_id=convert_or_none(device.site_id, int),
             device_name=device.device_name,
-            device_type=convert_or_none(device.device_type, enum_to_str),
-            device_subtype=convert_or_none(device.device_subtype, enum_to_str),
+            device_type=convert_or_none(device.device_type, permissive_enum_to_str),
+            device_subtype=convert_or_none(device.device_subtype, permissive_enum_to_str),
             device_description=device.device_description,
-            device_sample_rate=device.device_sample_rate,
+            device_sample_rate=convert_or_none(device.device_sample_rate, str),
             sending_ips=device.sending_ips,
             device_snmp_ip=device.device_snmp_ip,
             device_snmp_community=device.device_snmp_community,
             minimize_snmp=device.minimize_snmp,
-            device_bgp_type=convert_or_none(device.device_bgp_type, enum_to_str),
+            device_bgp_type=convert_or_none(device.device_bgp_type, permissive_enum_to_str),
             device_bgp_neighbor_ip=device.device_bgp_neighbor_ip,
             device_bgp_neighbor_ip6=device.device_bgp_neighbor_ip6,
             device_bgp_neighbor_asn=device.device_bgp_neighbor_asn,
@@ -255,7 +266,7 @@ class DevicePayload:
             device_bgp_password=device.device_bgp_password,
             use_bgp_device_id=convert_or_none(device.use_bgp_device_id, int),
             device_snmp_v3_conf=convert_or_none(device.device_snmp_v3_conf, SNMPv3ConfPayload.from_conf),
-            cdn_attr=convert_or_none(device.cdn_attr, enum_to_str),
+            cdn_attr=convert_or_none(device.cdn_attr, permissive_enum_to_str),
         )
 
     def to_device(self) -> Device:
@@ -266,7 +277,7 @@ class DevicePayload:
             device_name=self.device_name,
             device_type=DeviceType(self.device_type),
             device_subtype=DeviceSubtype(self.device_subtype),
-            device_sample_rate=self.device_sample_rate,
+            device_sample_rate=convert_or_none(self.device_sample_rate, int),
             sending_ips=self.sending_ips,
             device_description=self.device_description,
             device_snmp_ip=self.device_snmp_ip,
@@ -299,9 +310,9 @@ class GetResponse:
     device: DevicePayload
 
     @classmethod
-    def from_json(cls, json_string):
+    def from_json(cls, json_string: str):
         # for GET response the payload json is like: "device": {...}
-        dic = from_json(class_name=cls.__name__, json_string=json_string, root="device")
+        dic = dict_from_json(class_name=cls.__name__, json_string=json_string, root="device")
         return cls.from_dict(dic)
 
     @classmethod
@@ -317,9 +328,9 @@ class GetAllResponse:
     devices: List[GetResponse]
 
     @classmethod
-    def from_json(cls, json_string):
-        dic = from_json(class_name=cls.__name__, json_string=json_string, root="devices")
-        devices = [GetResponse.from_dict(item) for item in dic]
+    def from_json(cls, json_string: str):
+        items = list_from_json(class_name=cls.__name__, json_string=json_string, root="devices")
+        devices = [GetResponse.from_dict(item) for item in items]
         return cls(devices=devices)
 
     def to_devices(self) -> List[Device]:
@@ -445,7 +456,7 @@ class ApplyLabelsResponse:
 
     @classmethod
     def from_json(cls, json_string: str):
-        dic = from_json(class_name=cls.__name__, json_string=json_string)
+        dic = dict_from_json(class_name=cls.__name__, json_string=json_string)
         required_fields = ["id", "device_name", "labels"]
         validate_fields(class_name=cls.__name__, required_fields=required_fields, dic=dic)
         labels = [LabelPayload.from_dict(item) for item in dic["labels"]]
