@@ -2,7 +2,7 @@ from typing import Any, Callable
 
 
 class Cmd:
-    """ Command desing pattern - wrap method call into an object for easy queuing and retrying """
+    """ Command design pattern - wrap method call into an object for easy queuing and retrying """
 
     def __init__(self, method: Callable, **method_params) -> None:
         self._method = method
