@@ -1,9 +1,9 @@
 from http import HTTPStatus
 
-from kentik_api.api_resources.custom_dimensions_api import CustomDimensionsAPI
 from kentik_api.api_calls.api_call import APICallMethods
-from kentik_api.public.types import ID
+from kentik_api.api_resources.custom_dimensions_api import CustomDimensionsAPI
 from kentik_api.public.custom_dimension import CustomDimension, Populator
+from kentik_api.public.types import ID
 from tests.unit.stub_api_connector import StubAPIConnector
 
 
@@ -34,10 +34,11 @@ def test_create_custom_dimension_success() -> None:
     # then request properly formed
     assert connector.last_url_path == "/customdimension"
     assert connector.last_method == APICallMethods.POST
-    assert connector.last_payload is not None
-    assert connector.last_payload["name"] == "c_testapi_dimension_1"
-    assert connector.last_payload["display_name"] == "dimension_display_name"
-    assert connector.last_payload["type"] == "string"
+    assert connector.last_payload == {
+        "name": "c_testapi_dimension_1",
+        "display_name": "dimension_display_name",
+        "type": "string",
+    }
 
     # and response properly parsed
     assert created.id == ID(42)
@@ -163,8 +164,9 @@ def test_update_custom_dimension_success() -> None:
     # then request properly formed
     assert connector.last_url_path == f"/customdimension/{dimension_id}"
     assert connector.last_method == APICallMethods.PUT
-    assert connector.last_payload is not None
-    assert connector.last_payload["display_name"] == "dimension_display_name2"
+    assert connector.last_payload == {
+        "display_name": "dimension_display_name2",
+    }
 
     # and response properly parsed
     assert updated.id == ID(42)
@@ -337,28 +339,30 @@ def test_create_populator_success() -> None:
     # then request properly formed
     assert connector.last_url_path == f"/customdimension/{dimension_id}/populator"
     assert connector.last_method == APICallMethods.POST
-    assert connector.last_payload is not None
-    assert "populator" in connector.last_payload
-    assert connector.last_payload["populator"]["value"] == "testapi-dimension-value-1"
-    assert connector.last_payload["populator"]["direction"] == "DST"
-    assert connector.last_payload["populator"]["device_name"] == "device1,128.0.0.100"
-    assert connector.last_payload["populator"]["device_type"] == "device-type1"
-    assert connector.last_payload["populator"]["site"] == "site1,site2,site3"
-    assert connector.last_payload["populator"]["interface_name"] == "interface1,interface2"
-    assert connector.last_payload["populator"]["addr"] == "128.0.0.1/32,128.0.0.2/32"
-    assert connector.last_payload["populator"]["port"] == "1001,1002"
-    assert connector.last_payload["populator"]["tcp_flags"] == "160"
-    assert connector.last_payload["populator"]["protocol"] == "6,17"
-    assert connector.last_payload["populator"]["asn"] == "101,102"
-    assert connector.last_payload["populator"]["lasthop_as_name"] == "asn101,asn102"
-    assert connector.last_payload["populator"]["nexthop_asn"] == "201,202"
-    assert connector.last_payload["populator"]["nexthop_as_name"] == "asn201,asn202"
-    assert connector.last_payload["populator"]["nexthop"] == "128.0.200.1/32,128.0.200.2/32"
-    assert connector.last_payload["populator"]["bgp_aspath"] == "3001,3002"
-    assert connector.last_payload["populator"]["bgp_community"] == "401:499,501:599"
-    assert connector.last_payload["populator"]["mac"] == "FF:FF:FF:FF:FF:FA,FF:FF:FF:FF:FF:FF"
-    assert connector.last_payload["populator"]["country"] == "NL,SE"
-    assert connector.last_payload["populator"]["vlans"] == "2001,2002"
+    assert connector.last_payload == {
+        "populator": {
+            "value": "testapi-dimension-value-1",
+            "direction": "DST",
+            "device_name": "device1,128.0.0.100",
+            "device_type": "device-type1",
+            "site": "site1,site2,site3",
+            "interface_name": "interface1,interface2",
+            "addr": "128.0.0.1/32,128.0.0.2/32",
+            "port": "1001,1002",
+            "tcp_flags": "160",
+            "protocol": "6,17",
+            "asn": "101,102",
+            "lasthop_as_name": "asn101,asn102",
+            "nexthop_asn": "201,202",
+            "nexthop_as_name": "asn201,asn202",
+            "nexthop": "128.0.200.1/32,128.0.200.2/32",
+            "bgp_aspath": "3001,3002",
+            "bgp_community": "401:499,501:599",
+            "mac": "FF:FF:FF:FF:FF:FA,FF:FF:FF:FF:FF:FF",
+            "country": "NL,SE",
+            "vlans": "2001,2002",
+        }
+    }
 
     # and response properly parsed
     assert created.dimension_id == ID(24001)
@@ -435,28 +439,17 @@ def test_update_populator_success() -> None:
     # then request properly formed
     assert connector.last_url_path == f"/customdimension/{dimension_id}/populator/{populator_id}"
     assert connector.last_method == APICallMethods.PUT
-    assert connector.last_payload is not None
-    assert "populator" in connector.last_payload
-    assert connector.last_payload["populator"]["value"] == "testapi-dimension-value-3"
-    assert connector.last_payload["populator"]["direction"] == "SRC"
-    assert connector.last_payload["populator"]["device_type"] == "device-type3"
-    assert connector.last_payload["populator"]["site"] == "site3"
-    assert connector.last_payload["populator"]["interface_name"] == "interface3"
-    assert connector.last_payload["populator"]["tcp_flags"] == "12"
-    assert connector.last_payload["populator"]["protocol"] == "17"
-    assert "device_name" not in connector.last_payload["populator"]
-    assert "addr" not in connector.last_payload["populator"]
-    assert "port" not in connector.last_payload["populator"]
-    assert "asn" not in connector.last_payload["populator"]
-    assert "lasthop_as_name" not in connector.last_payload["populator"]
-    assert "nexthop_asn" not in connector.last_payload["populator"]
-    assert "nexthop_as_name" not in connector.last_payload["populator"]
-    assert "nexthop" not in connector.last_payload["populator"]
-    assert "bgp_aspath" not in connector.last_payload["populator"]
-    assert "bgp_community" not in connector.last_payload["populator"]
-    assert "mac" not in connector.last_payload["populator"]
-    assert "country" not in connector.last_payload["populator"]
-    assert "vlans" not in connector.last_payload["populator"]
+    assert connector.last_payload == {
+        "populator": {
+            "value": "testapi-dimension-value-3",
+            "direction": "SRC",
+            "device_type": "device-type3",
+            "site": "site3",
+            "interface_name": "interface3",
+            "tcp_flags": "12",
+            "protocol": "17",
+        },
+    }
 
     # and response properly parsed
     assert updated.id == ID(1510862280)
@@ -494,3 +487,194 @@ def test_delete_populator_success() -> None:
 
     # then response properly parsed
     assert delete_successful
+
+
+def test_get_and_update_custom_dimension_with_unknown_enum_fields() -> None:
+    # given
+    dimension_id = ID(42)
+    populator_id = ID(1510862280)
+
+    get_response_payload = """
+    {
+        "customDimension": {
+            "id": 42,
+            "name": "c_testapi_dimension_1",
+            "display_name": "dimension_display_name",
+            "type": "string",
+            "company_id": "74333",
+            "populators": [
+                {
+                    "id": 1510862280,
+                    "dimension_id": 42,
+                    "value": "testapi-dimension-value-3",
+                    "direction": "d_teapot",
+                    "addr_count": 0,
+                    "user": "144319",
+                    "created_date": "2020-12-15T07:55:23.911095Z",
+                    "updated_date": "2020-12-15T11:11:30.300681Z",
+                    "company_id": "74333",
+                    "mac_count": 0
+                }
+            ]
+        }
+    }"""
+    connector = StubAPIConnector(get_response_payload, HTTPStatus.OK)
+    custom_dimensions_api = CustomDimensionsAPI(connector)
+
+    # when custom dimension got
+    dimension = custom_dimensions_api.get(dimension_id)
+
+    # then get request properly formed
+    assert connector.last_url_path == f"/customdimension/{dimension_id}"
+    assert connector.last_method == APICallMethods.GET
+    assert connector.last_payload is None
+
+    # then get response properly parsed
+    check_dimension_one(dimension, expected_value="testapi-dimension-value-3", expected_direction="D_TEAPOT")
+
+    # given
+    update_response_payload = """
+    {
+        "populator": {
+            "id": 1510862280,
+            "dimension_id": 42,
+            "value": "updated_value",
+            "direction": "D_TEAPOT",
+            "addr_count": 0,
+            "user": "144319",
+            "created_date": "2020-12-15T07:55:23.911095Z",
+            "updated_date": "2020-12-15T11:11:30.300681Z",
+            "company_id": "74333",
+            "mac_count": 0
+        }
+    }"""
+    connector.response_text = update_response_payload
+
+    assert dimension.populators is not None
+    assert len(dimension.populators) == 1
+    populator = dimension.populators[0]
+
+    # when populator updated
+    populator.value = "updated_value"
+    populator = custom_dimensions_api.populators.update(populator)
+
+    # then update request properly formed
+    assert connector.last_url_path == f"/customdimension/{dimension_id}/populator/{populator_id}"
+    assert connector.last_method == APICallMethods.PUT
+    assert connector.last_payload == {
+        "populator": {
+            "value": "updated_value",
+            "direction": "D_TEAPOT",
+        },
+    }
+
+    # then update response properly parsed
+    check_populator_one(populator, expected_value="updated_value", expected_direction="D_TEAPOT")
+
+
+def test_get_and_update_custom_dimension_with_empty_enum_fields() -> None:
+    # given
+    dimension_id = ID(42)
+    populator_id = ID(1510862280)
+
+    get_response_payload = """
+    {
+        "customDimension": {
+            "id": 42,
+            "name": "c_testapi_dimension_1",
+            "display_name": "dimension_display_name",
+            "type": "string",
+            "company_id": "74333",
+            "populators": [
+                {
+                    "id": 1510862280,
+                    "dimension_id": 42,
+                    "value": "testapi-dimension-value-3",
+                    "direction": "",
+                    "addr_count": 0,
+                    "user": "144319",
+                    "created_date": "2020-12-15T07:55:23.911095Z",
+                    "updated_date": "2020-12-15T11:11:30.300681Z",
+                    "company_id": "74333",
+                    "mac_count": 0
+                }
+            ]
+        }
+    }"""
+    connector = StubAPIConnector(get_response_payload, HTTPStatus.OK)
+    custom_dimensions_api = CustomDimensionsAPI(connector)
+
+    # when custom dimension get
+    dimension = custom_dimensions_api.get(dimension_id)
+
+    # then get request properly formed
+    assert connector.last_url_path == f"/customdimension/{dimension_id}"
+    assert connector.last_method == APICallMethods.GET
+    assert connector.last_payload is None
+
+    # then get response properly parsed
+    check_dimension_one(dimension, expected_value="testapi-dimension-value-3", expected_direction="")
+
+    # given
+    update_response_payload = """
+    {
+        "populator": {
+            "id": 1510862280,
+            "dimension_id": 42,
+            "value": "updated_value",
+            "direction": "",
+            "addr_count": 0,
+            "user": "144319",
+            "created_date": "2020-12-15T07:55:23.911095Z",
+            "updated_date": "2020-12-15T11:11:30.300681Z",
+            "company_id": "74333",
+            "mac_count": 0
+        }
+    }"""
+    connector.response_text = update_response_payload
+
+    assert dimension.populators is not None
+    assert len(dimension.populators) == 1
+    populator = dimension.populators[0]
+
+    # when populator updated
+    populator.value = "updated_value"
+    populator = custom_dimensions_api.populators.update(populator)
+
+    # then update request properly formed
+    assert connector.last_url_path == f"/customdimension/{dimension_id}/populator/{populator_id}"
+    assert connector.last_method == APICallMethods.PUT
+    assert connector.last_payload == {
+        "populator": {
+            "value": "updated_value",
+            "direction": "",
+        },
+    }
+
+    # then update response properly parsed
+    check_populator_one(populator, expected_value="updated_value", expected_direction="")
+
+
+def check_dimension_one(dimension, expected_value, expected_direction) -> None:
+    assert dimension.id == ID(42)
+    assert dimension.name == "c_testapi_dimension_1"
+    assert dimension.display_name == "dimension_display_name"
+    assert dimension.type == "string"
+    assert dimension.company_id == ID(74333)
+
+    assert dimension.populators is not None
+    assert len(dimension.populators) == 1
+    check_populator_one(dimension.populators[0], expected_value=expected_value, expected_direction=expected_direction)
+
+
+def check_populator_one(populator, expected_value, expected_direction) -> None:
+    assert populator.id == ID(1510862280)
+    assert populator.dimension_id == ID(42)
+    assert populator.value == expected_value
+    assert populator.direction == expected_direction
+    assert populator.addr_count == 0
+    assert populator.user == "144319"
+    assert populator.created_date == "2020-12-15T07:55:23.911095Z"
+    assert populator.updated_date == "2020-12-15T11:11:30.300681Z"
+    assert populator.company_id == ID(74333)
+    assert populator.mac_count == 0
